@@ -156,6 +156,12 @@ def hide_fields_in_newer_versions(obj):
     if not api_utils.allow_storage_interface():
         obj.storage_interface = wsme.Unset
 
+    if not api_utils.allow_traits():
+        obj.traits = wsme.Unset
+
+    if not api_utils.allow_rescue_interface():
+        obj.rescue_interface = wsme.Unset
+
 
 def update_state_in_older_versions(obj):
     """Change provision state names for API backwards compatibility.
@@ -543,10 +549,10 @@ class NodeStatesController(rest.RestController):
 
     @METRICS.timer('NodeStatesController.provision')
     @expose.expose(None, types.uuid_or_name, wtypes.text,
-                   wtypes.text, types.jsontype,
+                   wtypes.text, types.jsontype, wtypes.text,
                    status_code=http_client.ACCEPTED)
     def provision(self, node_ident, target, configdrive=None,
-                  clean_steps=None):
+                  clean_steps=None, rescue_password=None):
         """Asynchronous trigger the provisioning of the node.
 
         This will set the target provision state of the node, and a
@@ -579,6 +585,9 @@ class NodeStatesController(rest.RestController):
                 'args': {'force': True} }
 
             This is required (and only valid) when target is "clean".
+        :param rescue_password: A string representing the password to be set
+            inside the rescue environment. This is required (and only valid),
+            when target is "rescue".
         :raises: NodeLocked (HTTP 409) if the node is currently locked.
         :raises: ClientSideError (HTTP 409) if the node is already being
                  provisioned.
@@ -631,6 +640,13 @@ class NodeStatesController(rest.RestController):
             raise wsme.exc.ClientSideError(
                 msg, status_code=http_client.BAD_REQUEST)
 
+        if (rescue_password is not None and
+            target != ir_states.VERBS['rescue']):
+            msg = (_('"rescue_password" is only valid when setting target '
+                     'provision state to %s') % ir_states.VERBS['rescue'])
+            raise wsme.exc.ClientSideError(
+                msg, status_code=http_client.BAD_REQUEST)
+
         # Note that there is a race condition. The node state(s) could change
         # by the time the RPC call is made and the TaskManager manager gets a
         # lock.
@@ -641,6 +657,18 @@ class NodeStatesController(rest.RestController):
                                                 rebuild=rebuild,
                                                 configdrive=configdrive,
                                                 topic=topic)
+        elif (target == ir_states.VERBS['unrescue']):
+            pecan.request.rpcapi.do_node_unrescue(
+                pecan.request.context, rpc_node.uuid, topic)
+        elif (target == ir_states.VERBS['rescue']):
+            if not (rescue_password and rescue_password.strip()):
+                msg = (_('A non-empty "rescue_password" is required when '
+                         'setting target provision state to %s') %
+                       ir_states.VERBS['rescue'])
+                raise wsme.exc.ClientSideError(
+                    msg, status_code=http_client.BAD_REQUEST)
+            pecan.request.rpcapi.do_node_rescue(
+                pecan.request.context, rpc_node.uuid, rescue_password, topic)
         elif target == ir_states.DELETED:
             pecan.request.rpcapi.do_node_tear_down(
                 pecan.request.context, rpc_node.uuid, topic)
@@ -686,6 +714,159 @@ def _check_clean_steps(clean_steps):
                                               exc)
 
 
+class Traits(base.APIBase):
+    """API representation of the traits for a node."""
+
+    traits = wtypes.ArrayType(str)
+    """node traits"""
+
+    @classmethod
+    def sample(cls):
+        traits = ["CUSTOM_TRAIT1", "CUSTOM_TRAIT2"]
+        return cls(traits=traits)
+
+
+def _get_chassis_uuid(node):
+    """Return the UUID of a node's chassis, or None.
+
+    :param node: a Node object.
+    :returns: the UUID of the node's chassis, or None if the node has no
+        chassis set.
+    """
+    if not node.chassis_id:
+        return
+    chassis = objects.Chassis.get_by_id(pecan.request.context, node.chassis_id)
+    return chassis.uuid
+
+
+def _make_trait_list(context, node_id, traits):
+    """Return a TraitList object for the specified node and traits.
+
+    The Trait objects will not be created in the database.
+
+    :param context: a request context.
+    :param node_id: the ID of a node.
+    :param traits: a list of trait strings to add to the TraitList.
+    :returns: a TraitList object.
+    """
+    trait_objs = [objects.Trait(context, node_id=node_id, trait=t)
+                  for t in traits]
+    return objects.TraitList(context, objects=trait_objs)
+
+
+class NodeTraitsController(rest.RestController):
+
+    def __init__(self, node_ident):
+        super(NodeTraitsController, self).__init__()
+        self.node_ident = node_ident
+
+    @METRICS.timer('NodeTraitsController.get_all')
+    @expose.expose(Traits)
+    def get_all(self):
+        """List node traits."""
+        cdict = pecan.request.context.to_policy_values()
+        policy.authorize('baremetal:node:traits:list', cdict, cdict)
+        node = api_utils.get_rpc_node(self.node_ident)
+        traits = objects.TraitList.get_by_node_id(pecan.request.context,
+                                                  node.id)
+        return Traits(traits=traits.get_trait_names())
+
+    @METRICS.timer('NodeTraitsController.put')
+    @expose.expose(None, wtypes.text, wtypes.ArrayType(str),
+                   status_code=http_client.NO_CONTENT)
+    def put(self, trait=None, traits=None):
+        """Add a trait to a node.
+
+        :param trait: String value; trait to add to a node, or None. Mutually
+            exclusive with 'traits'. If not None, adds this trait to the node.
+        :param traits: List of Strings; traits to set for a node, or None.
+            Mutually exclusive with 'trait'. If not None, replaces the node's
+            traits with this list.
+        """
+        context = pecan.request.context
+        cdict = context.to_policy_values()
+        policy.authorize('baremetal:node:traits:set', cdict, cdict)
+        node = api_utils.get_rpc_node(self.node_ident)
+
+        if (trait and traits is not None) or not (trait or traits is not None):
+            msg = _("A single node trait may be added via PUT "
+                    "/v1/nodes/<node identifier>/traits/<trait> with no body, "
+                    "or all node traits may be replaced via PUT "
+                    "/v1/nodes/<node identifier>/traits with the list of "
+                    "traits specified in the request body.")
+            raise exception.Invalid(msg)
+
+        if trait:
+            if pecan.request.body and pecan.request.json_body:
+                # Ensure PUT nodes/uuid1/traits/trait1 with a non-empty body
+                # fails.
+                msg = _("No body should be provided when adding a trait")
+                raise exception.Invalid(msg)
+            traits = [trait]
+            replace = False
+            new_traits = {t.trait for t in node.traits} | {trait}
+        else:
+            replace = True
+            new_traits = set(traits)
+
+        for trait in traits:
+            api_utils.validate_trait(trait)
+
+        # Update the node's traits to reflect the desired state.
+        node.traits = _make_trait_list(context, node.id, sorted(new_traits))
+        node.obj_reset_changes()
+        chassis_uuid = _get_chassis_uuid(node)
+        notify.emit_start_notification(context, node, 'update',
+                                       chassis_uuid=chassis_uuid)
+        with notify.handle_error_notification(context, node, 'update',
+                                              chassis_uuid=chassis_uuid):
+            topic = pecan.request.rpcapi.get_topic_for(node)
+            pecan.request.rpcapi.add_node_traits(
+                context, node.id, traits, replace=replace, topic=topic)
+        notify.emit_end_notification(context, node, 'update',
+                                     chassis_uuid=chassis_uuid)
+
+        if not replace:
+            # For single traits, set the HTTP Location Header.
+            url_args = '/'.join((self.node_ident, 'traits', trait))
+            pecan.response.location = link.build_url('nodes', url_args)
+
+    @METRICS.timer('NodeTraitsController.delete')
+    @expose.expose(None, wtypes.text,
+                   status_code=http_client.NO_CONTENT)
+    def delete(self, trait=None):
+        """Remove one or all traits from a node.
+
+        :param trait: String value; trait to remove from a node, or None. If
+                      None, all traits are removed.
+        """
+        context = pecan.request.context
+        cdict = context.to_policy_values()
+        policy.authorize('baremetal:node:traits:delete', cdict, cdict)
+        node = api_utils.get_rpc_node(self.node_ident)
+
+        if trait:
+            traits = [trait]
+            new_traits = {t.trait for t in node.traits} - {trait}
+        else:
+            traits = None
+            new_traits = set()
+
+        # Update the node's traits to reflect the desired state.
+        node.traits = _make_trait_list(context, node.id, sorted(new_traits))
+        node.obj_reset_changes()
+        chassis_uuid = _get_chassis_uuid(node)
+        notify.emit_start_notification(context, node, 'update',
+                                       chassis_uuid=chassis_uuid)
+        with notify.handle_error_notification(context, node, 'update',
+                                              chassis_uuid=chassis_uuid):
+            topic = pecan.request.rpcapi.get_topic_for(node)
+            pecan.request.rpcapi.remove_node_traits(
+                context, node.id, traits, topic=topic)
+        notify.emit_end_notification(context, node, 'update',
+                                     chassis_uuid=chassis_uuid)
+
+
 class Node(base.APIBase):
     """API representation of a bare metal node.
 
@@ -843,12 +1024,18 @@ class Node(base.APIBase):
     raid_interface = wsme.wsattr(wtypes.text)
     """The raid interface to be used for this node"""
 
+    rescue_interface = wsme.wsattr(wtypes.text)
+    """The rescue interface to be used for this node"""
+
     storage_interface = wsme.wsattr(wtypes.text)
     """The storage interface to be used for this node"""
 
     vendor_interface = wsme.wsattr(wtypes.text)
     """The vendor interface to be used for this node"""
 
+    traits = wtypes.ArrayType(str)
+    """The traits associated with this node"""
+
     # NOTE(deva): "conductor_affinity" shouldn't be presented on the
     #             API because it's an internal value. Don't add it here.
 
@@ -862,7 +1049,12 @@ class Node(base.APIBase):
             # Add fields we expose.
             if hasattr(self, k):
                 self.fields.append(k)
-                setattr(self, k, kwargs.get(k, wtypes.Unset))
+                # TODO(jroll) is there a less hacky way to do this?
+                if k == 'traits' and kwargs.get('traits') is not None:
+                    value = kwargs['traits'].get_trait_names()
+                else:
+                    value = kwargs.get(k, wtypes.Unset)
+                setattr(self, k, value)
 
         # NOTE(lucasagomes): chassis_id is an attribute created on-the-fly
         # by _set_chassis_uuid(), it needs to be present in the fields so
@@ -998,7 +1190,7 @@ class Node(base.APIBase):
                      deploy_interface=None, inspect_interface=None,
                      management_interface=None, power_interface=None,
                      raid_interface=None, vendor_interface=None,
-                     storage_interface=None)
+                     storage_interface=None, traits=[], rescue_interface=None)
         # NOTE(matty_dubs): The chassis_uuid getter() is based on the
         # _chassis_uuid variable:
         sample._chassis_uuid = 'edcad704-b2da-41d5-96d9-afd580ecfa12'
@@ -1258,13 +1450,15 @@ class NodesController(rest.RestController):
 
     invalid_sort_key_list = ['properties', 'driver_info', 'extra',
                              'instance_info', 'driver_internal_info',
-                             'clean_step', 'raid_config', 'target_raid_config']
+                             'clean_step', 'raid_config', 'target_raid_config',
+                             'traits']
 
     _subcontroller_map = {
         'ports': port.PortsController,
         'portgroups': portgroup.PortgroupsController,
         'vifs': NodeVIFController,
         'volume': volume.VolumeController,
+        'traits': NodeTraitsController,
     }
 
     @pecan.expose()
@@ -1280,6 +1474,11 @@ class NodesController(rest.RestController):
             (remainder[0] == 'vifs' and
                 not api_utils.allow_vifs_subcontroller())):
             pecan.abort(http_client.NOT_FOUND)
+        if remainder[0] == 'traits' and not api_utils.allow_traits():
+            # NOTE(mgoddard): Returning here will ensure we exhibit the
+            # behaviour of previous releases for microversions without this
+            # endpoint.
+            return
         subcontroller = self._subcontroller_map.get(remainder[0])
         if subcontroller:
             return subcontroller(node_ident=ident), remainder[1:]
@@ -1394,7 +1593,9 @@ class NodesController(rest.RestController):
         """Update rpc_node based on changed fields in a node.
 
         """
-        for field in objects.Node.fields:
+        # NOTE(mgoddard): Traits cannot be updated via a node PATCH.
+        fields = set(objects.Node.fields) - {'traits'}
+        for field in fields:
             try:
                 patch_val = getattr(node, field)
             except AttributeError:
@@ -1587,7 +1788,7 @@ class NodesController(rest.RestController):
         api_utils.check_allow_specify_fields(fields)
         api_utils.check_allowed_fields(fields)
 
-        rpc_node = api_utils.get_rpc_node(node_ident)
+        rpc_node = api_utils.get_rpc_node_with_suffix(node_ident)
         return Node.convert_with_links(rpc_node, fields=fields)
 
     @METRICS.timer('NodesController.post')
@@ -1622,6 +1823,15 @@ class NodesController(rest.RestController):
                 node.storage_interface is not wtypes.Unset):
             raise exception.NotAcceptable()
 
+        if node.traits is not wtypes.Unset:
+            msg = _("Cannot specify node traits on node creation. Traits must "
+                    "be set via the node traits API.")
+            raise exception.Invalid(msg)
+
+        if (not api_utils.allow_rescue_interface() and
+                node.rescue_interface is not wtypes.Unset):
+            raise exception.NotAcceptable()
+
         # NOTE(deva): get_topic_for checks if node.driver is in the hash ring
         #             and raises NoValidHost if it is not.
         #             We need to ensure that node has a UUID before it can
@@ -1693,7 +1903,17 @@ class NodesController(rest.RestController):
         if s_interface and not api_utils.allow_storage_interface():
             raise exception.NotAcceptable()
 
-        rpc_node = api_utils.get_rpc_node(node_ident)
+        traits = api_utils.get_patch_values(patch, '/traits')
+        if traits:
+            msg = _("Cannot update node traits via node patch. Node traits "
+                    "should be updated via the node traits API.")
+            raise exception.Invalid(msg)
+
+        r_interface = api_utils.get_patch_values(patch, '/rescue_interface')
+        if r_interface and not api_utils.allow_rescue_interface():
+            raise exception.NotAcceptable()
+
+        rpc_node = api_utils.get_rpc_node_with_suffix(node_ident)
 
         remove_inst_uuid_patch = [{'op': 'remove', 'path': '/instance_uuid'}]
         if rpc_node.maintenance and patch == remove_inst_uuid_patch:
@@ -1768,11 +1988,8 @@ class NodesController(rest.RestController):
         if self.from_chassis:
             raise exception.OperationNotPermitted()
 
-        rpc_node = api_utils.get_rpc_node(node_ident)
-        chassis_uuid = None
-        if rpc_node.chassis_id:
-            chassis_uuid = objects.Chassis.get_by_id(context,
-                                                     rpc_node.chassis_id).uuid
+        rpc_node = api_utils.get_rpc_node_with_suffix(node_ident)
+        chassis_uuid = _get_chassis_uuid(rpc_node)
         notify.emit_start_notification(context, rpc_node, 'delete',
                                        chassis_uuid=chassis_uuid)
         with notify.handle_error_notification(context, rpc_node, 'delete',
