@@ -33,6 +33,7 @@ from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
 from sqlalchemy.orm import joinedload
 from sqlalchemy import sql
 
+from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import profiler
@@ -75,8 +76,14 @@ def _wrap_session(session):
     return session
 
 
-def _get_node_query_with_tags():
-    return model_query(models.Node).options(joinedload('tags'))
+def _get_node_query_with_all():
+    """Return a query object for the Node model joined with all relevant fields.
+
+    :returns: a query object.
+    """
+    return (model_query(models.Node)
+            .options(joinedload('tags'))
+            .options(joinedload('traits')))
 
 
 def model_query(model, *args, **kwargs):
@@ -270,7 +277,7 @@ class Connection(api.Connection):
 
     def get_node_list(self, filters=None, limit=None, marker=None,
                       sort_key=None, sort_dir=None):
-        query = _get_node_query_with_tags()
+        query = _get_node_query_with_all()
         query = self._add_nodes_filters(query, filters)
         return _paginate_query(models.Node, limit, marker,
                                sort_key, sort_dir, query)
@@ -278,7 +285,7 @@ class Connection(api.Connection):
     @oslo_db_api.retry_on_deadlock
     def reserve_node(self, tag, node_id):
         with _session_for_write():
-            query = _get_node_query_with_tags()
+            query = _get_node_query_with_all()
             query = add_identity_filter(query, node_id)
             # be optimistic and assume we usually create a reservation
             count = query.filter_by(reservation=None).update(
@@ -348,12 +355,15 @@ class Connection(api.Connection):
                         node=values['uuid'])
                 raise exception.NodeAlreadyExists(uuid=values['uuid'])
             # Set tags & traits to [] for new created node
+            # NOTE(mgoddard): We need to set the tags and traits fields in the
+            # session context, otherwise SQLAlchemy will try and fail to lazy
+            # load the attributes, resulting in an exception being raised.
             node['tags'] = []
             node['traits'] = []
-            return node
+        return node
 
     def get_node_by_id(self, node_id):
-        query = _get_node_query_with_tags()
+        query = _get_node_query_with_all()
         query = query.filter_by(id=node_id)
         try:
             return query.one()
@@ -361,7 +371,7 @@ class Connection(api.Connection):
             raise exception.NodeNotFound(node=node_id)
 
     def get_node_by_uuid(self, node_uuid):
-        query = _get_node_query_with_tags()
+        query = _get_node_query_with_all()
         query = query.filter_by(uuid=node_uuid)
         try:
             return query.one()
@@ -369,7 +379,7 @@ class Connection(api.Connection):
             raise exception.NodeNotFound(node=node_uuid)
 
     def get_node_by_name(self, node_name):
-        query = _get_node_query_with_tags()
+        query = _get_node_query_with_all()
         query = query.filter_by(name=node_name)
         try:
             return query.one()
@@ -380,7 +390,7 @@ class Connection(api.Connection):
         if not uuidutils.is_uuid_like(instance):
             raise exception.InvalidUUID(uuid=instance)
 
-        query = _get_node_query_with_tags()
+        query = _get_node_query_with_all()
         query = query.filter_by(instance_uuid=instance)
 
         try:
@@ -457,6 +467,8 @@ class Connection(api.Connection):
     @oslo_db_api.retry_on_deadlock
     def _do_update_node(self, node_id, values):
         with _session_for_write():
+            # NOTE(mgoddard): Don't issue a joined query for the update as this
+            # does not work with PostgreSQL.
             query = model_query(models.Node)
             query = add_identity_filter(query, node_id)
             try:
@@ -478,7 +490,11 @@ class Connection(api.Connection):
                     values['inspection_started_at'] = None
 
             ref.update(values)
-        return ref
+
+            # Return the updated node model joined with all relevant fields.
+            query = _get_node_query_with_all()
+            query = add_identity_filter(query, node_id)
+            return query.one()
 
     def get_port_by_id(self, port_id):
         query = model_query(models.Port).filter_by(id=port_id)
@@ -968,11 +984,13 @@ class Connection(api.Connection):
                 raise exception.NodeTagNotFound(node_id=node_id, tag=tag)
 
     def node_tag_exists(self, node_id, tag):
+        self._check_node_exists(node_id)
         q = model_query(models.NodeTag).filter_by(node_id=node_id, tag=tag)
         return model_query(q.exists()).scalar()
 
     def get_node_by_port_addresses(self, addresses):
-        q = model_query(models.Node).distinct().join(models.Port)
+        q = _get_node_query_with_all()
+        q = q.distinct().join(models.Port)
         q = q.filter(models.Port.address.in_(addresses))
 
         try:
@@ -1281,6 +1299,82 @@ class Connection(api.Connection):
 
         return total_to_migrate, total_migrated
 
+    @oslo_db_api.retry_on_deadlock
+    def migrate_to_hardware_types(self, context, max_count,
+                                  reset_unsupported_interfaces=False):
+        """Migrate nodes from classic drivers to hardware types.
+
+        Go through all nodes with a classic driver and try to migrate them to
+        a corresponding hardware type and a correct set of hardware interfaces.
+
+        If migration is not possible for any reason (e.g. the target hardware
+        type is not enabled), the nodes are skipped. An operator is expected to
+        correct the configuration and either rerun online_data_migration or
+        migrate the nodes manually.
+
+        :param context: the admin context (not used)
+        :param max_count: The maximum number of objects to migrate. Must be
+                          >= 0. If zero, all the objects will be migrated.
+        :param reset_unsupported_interfaces: whether to reset unsupported
+            optional interfaces to their no-XXX versions.
+        :returns: A 2-tuple, 1. the total number of objects that need to be
+                  migrated (at the beginning of this call) and 2. the number
+                  of migrated objects.
+        """
+        reset_unsupported_interfaces = strutils.bool_from_string(
+            reset_unsupported_interfaces, strict=True)
+
+        drivers = driver_factory.classic_drivers_to_migrate()
+
+        total_to_migrate = (model_query(models.Node)
+                            .filter(models.Node.driver.in_(list(drivers)))
+                            .count())
+
+        total_migrated = 0
+        for driver, driver_cls in drivers.items():
+            if max_count and total_migrated >= max_count:
+                return total_to_migrate, total_migrated
+
+            # UPDATE with LIMIT seems to be a MySQL-only feature, so first
+            # fetch the required number of Node IDs, then update them.
+            query = model_query(models.Node.id).filter_by(driver=driver)
+            if max_count:
+                query = query.limit(max_count - total_migrated)
+            ids = [obj.id for obj in query]
+            if not ids:
+                continue
+
+            delta = driver_factory.calculate_migration_delta(
+                driver, driver_cls, reset_unsupported_interfaces)
+            if delta is None:
+                # NOTE(dtantsur): mark unsupported nodes as migrated. Otherwise
+                # calling online_data_migration without --max-count will result
+                # in a infinite loop.
+                total_migrated += len(ids)
+                continue
+
+            # UPDATE with LIMIT seems to be a MySQL-only feature, so first
+            # fetch the required number of Node IDs, then update them.
+            query = model_query(models.Node.id).filter_by(driver=driver)
+            if max_count:
+                query = query.limit(max_count - total_migrated)
+            ids = [obj.id for obj in query]
+            if not ids:
+                LOG.debug('No nodes with driver %s', driver)
+                continue
+
+            LOG.info('Migrating nodes with driver %(drv)s to %(delta)s',
+                     {'drv': driver, 'delta': delta})
+
+            with _session_for_write():
+                num_migrated = (model_query(models.Node)
+                                .filter_by(driver=driver)
+                                .filter(models.Node.id.in_(ids))
+                                .update(delta, synchronize_session=False))
+                total_migrated += num_migrated
+
+        return total_to_migrate, total_migrated
+
     @staticmethod
     def _verify_max_traits_per_node(node_id, num_traits):
         """Verify that an operation would not exceed the per-node trait limit.
@@ -1300,7 +1394,7 @@ class Connection(api.Connection):
                 max_traits=MAX_TRAITS_PER_NODE)
 
     @oslo_db_api.retry_on_deadlock
-    def set_node_traits(self, node_id, traits):
+    def set_node_traits(self, node_id, traits, version):
         # Remove duplicate traits
         traits = set(traits)
 
@@ -1311,7 +1405,8 @@ class Connection(api.Connection):
             self.unset_node_traits(node_id)
             node_traits = []
             for trait in traits:
-                node_trait = models.NodeTrait(trait=trait, node_id=node_id)
+                node_trait = models.NodeTrait(trait=trait, node_id=node_id,
+                                              version=version)
                 session.add(node_trait)
                 node_traits.append(node_trait)
 
@@ -1331,8 +1426,9 @@ class Connection(api.Connection):
         return result
 
     @oslo_db_api.retry_on_deadlock
-    def add_node_trait(self, node_id, trait):
-        node_trait = models.NodeTrait(trait=trait, node_id=node_id)
+    def add_node_trait(self, node_id, trait, version):
+        node_trait = models.NodeTrait(trait=trait, node_id=node_id,
+                                      version=version)
 
         self._check_node_exists(node_id)
         try:
