@@ -49,11 +49,13 @@ class BaseDriver(object):
     the interfaces are appropriate.
     """
 
-    supported = True
+    supported = False
     """Indicates if a driver is supported.
 
     This will be set to False for drivers which are untested in first- or
     third-party CI, or in the process of being deprecated.
+
+    All classic drivers are now deprecated, and thus unsupported.
     """
 
     # NOTE(jlvillal): These should be tuples to help prevent child classes from
@@ -147,6 +149,18 @@ class BaseDriver(object):
                 properties.update(iface.get_properties())
         return properties
 
+    @classmethod
+    def to_hardware_type(cls):
+        """Return corresponding hardware type and hardware interfaces.
+
+        :returns: a tuple with two items:
+
+            * new driver field - the target hardware type
+            * dictionary containing interfaces to update, e.g.
+              {'deploy': 'iscsi', 'power': 'ipmitool'}
+        """
+        raise NotImplementedError()
+
 
 class BareDriver(BaseDriver):
     """A bare driver object which will have interfaces attached later.
@@ -422,7 +436,7 @@ class BootInterface(BaseInterface):
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of Ironic ramdisk.
 
-        This method prepares the boot of the deploy ramdisk after
+        This method prepares the boot of the deploy or rescue ramdisk after
         reading relevant information from the node's database.
 
         :param task: a task from TaskManager.
@@ -444,7 +458,7 @@ class BootInterface(BaseInterface):
         """Cleans up the boot of ironic ramdisk.
 
         This method cleans up the environment that was setup for booting the
-        deploy ramdisk.
+        deploy or rescue ramdisk.
 
         :param task: a task from TaskManager.
         :returns: None
@@ -1017,6 +1031,10 @@ class NetworkInterface(BaseInterface):
     def vif_detach(self, task, vif_id):
         """Detach a virtual network interface from a node
 
+        Warning: This method is called by the conductor as a shared
+        task, to ensure that a vif can be detached during a long-lived lock.
+        Such as those that occur when a node is being unprovisioned.
+
         :param task: A TaskManager instance.
         :param vif_id: A VIF ID to detach
         :raises: NetworkError, VifNotAttached
@@ -1036,8 +1054,9 @@ class NetworkInterface(BaseInterface):
         """Returns the currently used VIF associated with port or portgroup
 
         We are booting the node only in one network at a time, and presence of
-        cleaning_vif_port_id means we're doing cleaning, of
-        provisioning_vif_port_id - provisioning.
+        cleaning_vif_port_id means we're doing cleaning,
+        of provisioning_vif_port_id - provisioning,
+        of rescuing_vif_port_id - rescuing.
         Otherwise it's a tenant network.
 
         :param task: A TaskManager instance.
@@ -1092,6 +1111,38 @@ class NetworkInterface(BaseInterface):
         :raises: NetworkError
         """
 
+    def validate_rescue(self, task):
+        """Validates the network interface for rescue operation.
+
+        :param task: a TaskManager instance.
+        :raises: InvalidParameterValue, if the network interface configuration
+            is invalid.
+        :raises: MissingParameterValue, if some parameters are missing.
+        """
+        pass
+
+    def add_rescuing_network(self, task):
+        """Add the rescuing network to the node.
+
+        :param task: A TaskManager instance.
+        :returns: a dictionary in the form {port.uuid: neutron_port['id']}
+        :raises: NetworkError
+        :raises: InvalidParameterValue, if the network interface configuration
+            is invalid.
+        """
+        return {}
+
+    def remove_rescuing_network(self, task):
+        """Removes the rescuing network from a node.
+
+        :param task: A TaskManager instance.
+        :raises: NetworkError
+        :raises: InvalidParameterValue, if the network interface configuration
+            is invalid.
+        :raises: MissingParameterValue, if some parameters are missing.
+        """
+        pass
+
 
 @six.add_metaclass(abc.ABCMeta)
 class StorageInterface(BaseInterface):
@@ -1135,9 +1186,9 @@ def _validate_argsinfo(argsinfo):
     :param argsinfo: a dictionary of keyword arguments where key is the name of
         the argument and value is a dictionary as follows::
 
-            ‘description’: <description>. Required. This should include
+            'description': <description>. Required. This should include
                            possible values.
-            ‘required’: Boolean. Optional; default is False. True if this
+            'required': Boolean. Optional; default is False. True if this
                         argument is required.  If so, it must be specified in
                         the clean request; false if it is optional.
     :raises InvalidParameterValue: if any of the arguments are invalid
@@ -1229,9 +1280,9 @@ def clean_step(priority, abortable=False, argsinfo=None):
     :param argsinfo: a dictionary of keyword arguments where key is the name of
         the argument and value is a dictionary as follows::
 
-            ‘description’: <description>. Required. This should include
+            'description': <description>. Required. This should include
                            possible values.
-            ‘required’: Boolean. Optional; default is False. True if this
+            'required': Boolean. Optional; default is False. True if this
                         argument is required.  If so, it must be specified in
                         the clean request; false if it is optional.
     :raises InvalidParameterValue: if any of the arguments are invalid
