@@ -362,8 +362,9 @@ class VIFPortIDMixin(object):
         """Returns the currently used VIF associated with port or portgroup
 
         We are booting the node only in one network at a time, and presence of
-        cleaning_vif_port_id means we're doing cleaning, of
-        provisioning_vif_port_id - provisioning.
+        cleaning_vif_port_id means we're doing cleaning,
+        of provisioning_vif_port_id - provisioning,
+        of rescuing_vif_port_id - rescuing.
         Otherwise it's a tenant network
 
         :param task: A TaskManager instance.
@@ -373,6 +374,7 @@ class VIFPortIDMixin(object):
 
         return (p_obj.internal_info.get('cleaning_vif_port_id') or
                 p_obj.internal_info.get('provisioning_vif_port_id') or
+                p_obj.internal_info.get('rescuing_vif_port_id') or
                 self._get_vif_id_by_port_like_obj(p_obj) or None)
 
 
@@ -579,8 +581,8 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # attached, and fail if not.
         port_like_obj = self._get_port_like_obj_by_vif_id(task, vif_id)
 
-        self._clear_vif_from_port_like_obj(port_like_obj)
-
         # NOTE(vsaienko) allow to unplug VIFs from ACTIVE instance.
         if task.node.provision_state == states.ACTIVE:
             neutron.unbind_neutron_port(vif_id, context=task.context)
+
+        self._clear_vif_from_port_like_obj(port_like_obj)
