@@ -14,6 +14,7 @@
 
 import mock
 from oslo_utils import uuidutils
+import stevedore
 from stevedore import named
 
 from ironic.common import driver_factory
@@ -30,7 +31,7 @@ from ironic.tests.unit.objects import utils as obj_utils
 
 
 class FakeEp(object):
-    name = 'fake'
+    name = 'fake-hardware'
 
 
 class DriverLoadTestCase(db_base.DbTestCase):
@@ -44,52 +45,62 @@ class DriverLoadTestCase(db_base.DbTestCase):
                                                driver='aaa', reason='bbb'))
 
     def test_driver_load_error_if_driver_enabled(self):
-        self.config(enabled_drivers=['fake'])
+        self.config(enabled_hardware_types=['fake-hardware'])
         with mock.patch.object(named.NamedExtensionManager,
                                '__init__', self._fake_init_driver_err):
             self.assertRaises(
                 exception.DriverLoadError,
-                driver_factory.DriverFactory._init_extension_manager)
+                driver_factory.HardwareTypesFactory._init_extension_manager)
 
     def test_wrap_in_driver_load_error_if_driver_enabled(self):
-        self.config(enabled_drivers=['fake'])
+        self.config(enabled_hardware_types=['fake-hardware'])
         with mock.patch.object(named.NamedExtensionManager,
                                '__init__', self._fake_init_name_err):
             self.assertRaises(
                 exception.DriverLoadError,
-                driver_factory.DriverFactory._init_extension_manager)
+                driver_factory.HardwareTypesFactory._init_extension_manager)
 
     @mock.patch.object(named.NamedExtensionManager, 'names',
                        autospec=True)
     def test_no_driver_load_error_if_driver_disabled(self, mock_em):
-        self.config(enabled_drivers=[])
+        self.config(enabled_hardware_types=[])
         with mock.patch.object(named.NamedExtensionManager,
                                '__init__', self._fake_init_driver_err):
-            driver_factory.DriverFactory._init_extension_manager()
+            driver_factory.HardwareTypesFactory._init_extension_manager()
             self.assertEqual(1, mock_em.call_count)
 
     @mock.patch.object(driver_factory.LOG, 'warning', autospec=True)
     def test_driver_duplicated_entry(self, mock_log):
-        self.config(enabled_drivers=['fake', 'fake'])
-        driver_factory.DriverFactory._init_extension_manager()
+        self.config(enabled_hardware_types=['fake-hardware',
+                                            'fake-hardware'])
+        driver_factory.HardwareTypesFactory._init_extension_manager()
         self.assertEqual(
-            ['fake'], driver_factory.DriverFactory._extension_manager.names())
+            ['fake-hardware'],
+            driver_factory.HardwareTypesFactory._extension_manager.names())
         self.assertTrue(mock_log.called)
 
     @mock.patch.object(driver_factory.LOG, 'warning', autospec=True)
     def test_driver_empty_entry(self, mock_log):
-        self.config(enabled_drivers=['fake', ''])
-        driver_factory.DriverFactory._init_extension_manager()
+        self.config(enabled_hardware_types=['fake-hardware', ''])
+        driver_factory.HardwareTypesFactory._init_extension_manager()
         self.assertEqual(
-            ['fake'], driver_factory.DriverFactory._extension_manager.names())
+            ['fake-hardware'],
+            driver_factory.HardwareTypesFactory._extension_manager.names())
         self.assertTrue(mock_log.called)
 
     @mock.patch.object(driver_factory, '_warn_if_unsupported', autospec=True)
     def test_driver_init_checks_unsupported(self, mock_warn):
+        self.config(enabled_hardware_types=['fake-hardware'])
+        driver_factory.HardwareTypesFactory._init_extension_manager()
+        self.assertEqual(
+            ['fake-hardware'],
+            driver_factory.HardwareTypesFactory._extension_manager.names())
+        self.assertTrue(mock_warn.called)
+
+    @mock.patch.object(driver_factory.LOG, 'warning', autospec=True)
+    def test_classic_drivers_unsupported(self, mock_warn):
         self.config(enabled_drivers=['fake'])
         driver_factory.DriverFactory._init_extension_manager()
-        self.assertEqual(
-            ['fake'], driver_factory.DriverFactory._extension_manager.names())
         self.assertTrue(mock_warn.called)
 
     def test_build_driver_for_task(self):
@@ -102,6 +113,7 @@ class DriverLoadTestCase(db_base.DbTestCase):
                 else:
                     self.assertIsNotNone(impl)
 
+    @mock.patch.object(drivers_base.BaseDriver, 'supported', True)
     @mock.patch.object(driver_factory, '_attach_interfaces_to_driver',
                        autospec=True)
     @mock.patch.object(driver_factory.LOG, 'warning', autospec=True)
@@ -468,7 +480,8 @@ class CheckAndUpdateNodeInterfacesTestCase(db_base.DbTestCase):
 class DefaultInterfaceTestCase(db_base.DbTestCase):
     def setUp(self):
         super(DefaultInterfaceTestCase, self).setUp()
-        self.config(enabled_hardware_types=['manual-management'])
+        self.config(enabled_hardware_types=['manual-management'],
+                    enabled_drivers=['fake'])
         self.driver = driver_factory.get_hardware_type('manual-management')
 
     def test_from_config(self):
@@ -481,16 +494,27 @@ class DefaultInterfaceTestCase(db_base.DbTestCase):
         iface = driver_factory.default_interface(self.driver, 'storage')
         self.assertEqual('noop', iface)
 
+    def test_network_from_additional_defaults_hardware_type(self):
+        self.config(default_network_interface=None)
+        self.config(dhcp_provider='none', group='dhcp')
+        self.config(enabled_network_interfaces=['neutron'])
+        iface = driver_factory.default_interface(self.driver, 'network')
+        self.assertEqual('neutron', iface)
+
     def test_network_from_additional_defaults(self):
         self.config(default_network_interface=None)
         self.config(dhcp_provider='none', group='dhcp')
-        iface = driver_factory.default_interface(self.driver, 'network')
+        iface = driver_factory.default_interface(
+            driver_factory.get_driver_or_hardware_type('fake'),
+            'network')
         self.assertEqual('noop', iface)
 
     def test_network_from_additional_defaults_neutron_dhcp(self):
         self.config(default_network_interface=None)
         self.config(dhcp_provider='neutron', group='dhcp')
-        iface = driver_factory.default_interface(self.driver, 'network')
+        iface = driver_factory.default_interface(
+            driver_factory.get_driver_or_hardware_type('fake'),
+            'network')
         self.assertEqual('flat', iface)
 
     def test_calculated_with_one(self):
@@ -796,3 +820,78 @@ class HardwareTypeLoadTestCase(db_base.DbTestCase):
 
     def test_enabled_supported_interfaces_non_default(self):
         self._test_enabled_supported_interfaces(True)
+
+
+class ClassicDriverMigrationTestCase(base.TestCase):
+
+    def setUp(self):
+        super(ClassicDriverMigrationTestCase, self).setUp()
+        self.driver_cls = mock.Mock(spec=['to_hardware_type'])
+        self.driver_cls2 = mock.Mock(spec=['to_hardware_type'])
+        self.new_ifaces = {
+            'console': 'new-console',
+            'inspect': 'new-inspect'
+        }
+
+        self.driver_cls.to_hardware_type.return_value = ('hw-type',
+                                                         self.new_ifaces)
+        self.ext = mock.Mock(plugin=self.driver_cls)
+        self.ext.name = 'drv1'
+        self.ext2 = mock.Mock(plugin=self.driver_cls2)
+        self.ext2.name = 'drv2'
+        self.config(enabled_hardware_types=['hw-type'],
+                    enabled_console_interfaces=['no-console', 'new-console'],
+                    enabled_inspect_interfaces=['no-inspect', 'new-inspect'],
+                    enabled_raid_interfaces=['no-raid'],
+                    enabled_rescue_interfaces=['no-rescue'],
+                    enabled_vendor_interfaces=['no-vendor'])
+
+    def test_calculate_migration_delta(self):
+        delta = driver_factory.calculate_migration_delta(
+            'drv', self.driver_cls, False)
+        self.assertEqual({'driver': 'hw-type',
+                          'console_interface': 'new-console',
+                          'inspect_interface': 'new-inspect',
+                          'raid_interface': 'no-raid',
+                          'rescue_interface': 'no-rescue',
+                          'vendor_interface': 'no-vendor'},
+                         delta)
+
+    def test_calculate_migration_delta_not_implemeted(self):
+        self.driver_cls.to_hardware_type.side_effect = NotImplementedError()
+        delta = driver_factory.calculate_migration_delta(
+            'drv', self.driver_cls, False)
+        self.assertIsNone(delta)
+
+    def test_calculate_migration_delta_unsupported_hw_type(self):
+        self.driver_cls.to_hardware_type.return_value = ('hw-type2',
+                                                         self.new_ifaces)
+        delta = driver_factory.calculate_migration_delta(
+            'drv', self.driver_cls, False)
+        self.assertIsNone(delta)
+
+    def test__calculate_migration_delta_unsupported_interface(self):
+        self.new_ifaces['inspect'] = 'unsupported inspect'
+        delta = driver_factory.calculate_migration_delta(
+            'drv', self.driver_cls, False)
+        self.assertIsNone(delta)
+
+    def test_calculate_migration_delta_unsupported_interface_reset(self):
+        self.new_ifaces['inspect'] = 'unsupported inspect'
+        delta = driver_factory.calculate_migration_delta(
+            'drv', self.driver_cls, True)
+        self.assertEqual({'driver': 'hw-type',
+                          'console_interface': 'new-console',
+                          'inspect_interface': 'no-inspect',
+                          'raid_interface': 'no-raid',
+                          'rescue_interface': 'no-rescue',
+                          'vendor_interface': 'no-vendor'},
+                         delta)
+
+    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
+    def test_classic_drivers_to_migrate(self, mock_ext_mgr):
+        mock_ext_mgr.return_value.__iter__.return_value = iter([self.ext,
+                                                                self.ext2])
+        self.assertEqual({'drv1': self.driver_cls,
+                          'drv2': self.driver_cls2},
+                         driver_factory.classic_drivers_to_migrate())
