@@ -31,7 +31,9 @@ from ironic.tests.unit.db import utils
 class DbNodeTestCase(base.DbTestCase):
 
     def test_create_node(self):
-        utils.create_test_node()
+        node = utils.create_test_node()
+        self.assertEqual([], node.tags)
+        self.assertEqual([], node.traits)
 
     def test_create_node_with_tags(self):
         self.assertRaises(exception.InvalidParameterValue,
@@ -66,27 +68,39 @@ class DbNodeTestCase(base.DbTestCase):
     def test_get_node_by_id(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_id(node.id)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_uuid(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_uuid(node.uuid)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_name(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         res = self.dbapi.get_node_by_name(node.name)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertEqual(node.name, res.name)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_that_does_not_exist(self):
         self.assertRaises(exception.NodeNotFound,
@@ -233,6 +247,7 @@ class DbNodeTestCase(base.DbTestCase):
         six.assertCountEqual(self, uuids, res_uuids)
         for r in res:
             self.assertEqual([], r.tags)
+            self.assertEqual([], r.traits)
 
     def test_get_node_list_with_filters(self):
         ch1 = utils.create_test_chassis(uuid=uuidutils.generate_uuid())
@@ -289,10 +304,14 @@ class DbNodeTestCase(base.DbTestCase):
         node = utils.create_test_node(
             instance_uuid='12345678-9999-0000-aaaa-123456789012')
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
 
         res = self.dbapi.get_node_by_instance(node.instance_uuid)
         self.assertEqual(node.uuid, res.uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
     def test_get_node_by_instance_wrong_uuid(self):
         utils.create_test_node(
@@ -353,7 +372,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         self.assertTrue(self.dbapi.node_tag_exists(node.id, tag.tag))
         self.dbapi.destroy_node(node.id)
-        self.assertFalse(self.dbapi.node_tag_exists(node.id, tag.tag))
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_tag_exists, node.id, tag.tag)
 
     def test_tags_get_destroyed_after_destroying_a_node_by_uuid(self):
         node = utils.create_test_node()
@@ -362,7 +382,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         self.assertTrue(self.dbapi.node_tag_exists(node.id, tag.tag))
         self.dbapi.destroy_node(node.uuid)
-        self.assertFalse(self.dbapi.node_tag_exists(node.id, tag.tag))
+        self.assertRaises(exception.NodeNotFound,
+                          self.dbapi.node_tag_exists, node.id, tag.tag)
 
     def test_volume_connector_get_destroyed_after_destroying_a_node(self):
         node = utils.create_test_node()
@@ -433,6 +454,30 @@ class DbNodeTestCase(base.DbTestCase):
 
         res = self.dbapi.update_node(node.id, {'extra': new_extra})
         self.assertEqual(new_extra, res.extra)
+        self.assertEqual([], res.tags)
+        self.assertEqual([], res.traits)
+
+    def test_update_node_with_tags(self):
+        node = utils.create_test_node()
+        tag = utils.create_test_node_tag(node_id=node.id)
+
+        old_extra = node.extra
+        new_extra = {'foo': 'bar'}
+        self.assertNotEqual(old_extra, new_extra)
+
+        res = self.dbapi.update_node(node.id, {'extra': new_extra})
+        self.assertEqual([tag.tag], [t.tag for t in res.tags])
+
+    def test_update_node_with_traits(self):
+        node = utils.create_test_node()
+        trait = utils.create_test_node_trait(node_id=node.id)
+
+        old_extra = node.extra
+        new_extra = {'foo': 'bar'}
+        self.assertNotEqual(old_extra, new_extra)
+
+        res = self.dbapi.update_node(node.id, {'extra': new_extra})
+        self.assertEqual([trait.trait], [t.trait for t in res.traits])
 
     def test_update_node_not_found(self):
         node_uuid = uuidutils.generate_uuid()
@@ -515,6 +560,8 @@ class DbNodeTestCase(base.DbTestCase):
     def test_reserve_node(self):
         node = utils.create_test_node()
         self.dbapi.set_node_tags(node.id, ['tag1', 'tag2'])
+        utils.create_test_node_traits(node_id=node.id,
+                                      traits=['trait1', 'trait2'])
         uuid = node.uuid
 
         r1 = 'fake-reservation'
@@ -522,6 +569,8 @@ class DbNodeTestCase(base.DbTestCase):
         # reserve the node
         res = self.dbapi.reserve_node(r1, uuid)
         self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertItemsEqual(['trait1', 'trait2'],
+                              [trait.trait for trait in res.traits])
 
         # check reservation
         res = self.dbapi.get_node_by_uuid(uuid)
@@ -648,6 +697,7 @@ class DbNodeTestCase(base.DbTestCase):
 
         res = self.dbapi.get_node_by_port_addresses(addresses)
         self.assertEqual(node.uuid, res.uuid)
+        self.assertEqual([], res.traits)
 
     def test_get_node_by_port_addresses_not_found(self):
         node = utils.create_test_node(
