@@ -626,7 +626,7 @@ class TestVifPortIDMixin(db_base.DbTestCase):
 
     def test_get_current_vif_internal_info_provisioning(self):
         internal_info = {'provisioning_vif_port_id': 'foo',
-                         'vif_port_id': 'bar'}
+                         'tenant_vif_port_id': 'bar'}
         self.port.internal_info = internal_info
         self.port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -641,6 +641,15 @@ class TestVifPortIDMixin(db_base.DbTestCase):
             vif = self.interface.get_current_vif(task, self.port)
             self.assertEqual('bar', vif)
 
+    def test_get_current_vif_internal_info_rescuing(self):
+        internal_info = {'rescuing_vif_port_id': 'foo',
+                         'tenant_vif_port_id': 'bar'}
+        self.port.internal_info = internal_info
+        self.port.save()
+        with task_manager.acquire(self.context, self.node.id) as task:
+            vif = self.interface.get_current_vif(task, self.port)
+            self.assertEqual('foo', vif)
+
     def test_get_current_vif_none(self):
         internal_info = extra = {}
         self.port.internal_info = internal_info
@@ -932,12 +941,28 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         self.node.save()
         mock_get.return_value = self.port
         mock_unp.side_effect = exception.NetworkError
-        with task_manager.acquire(self.context, self.node.id) as task:
+        with task_manager.acquire(self.context, self.node.id,
+                                  shared=True) as task:
             self.assertRaises(exception.NetworkError,
                               self.interface.vif_detach, task, 'fake_vif_id')
             mock_unp.assert_called_once_with('fake_vif_id',
                                              context=task.context)
         mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_clear.assert_not_called()
+
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    def test_vif_detach_deleting_node_success(self, mock_get, mock_unp,
+                                              mock_clear):
+        self.node.provision_state = states.DELETING
+        self.node.save()
+        mock_get.return_value = self.port
+        with task_manager.acquire(self.context, self.node.id,
+                                  shared=True) as task:
+            self.interface.vif_detach(task, 'fake_vif_id')
+        self.assertFalse(mock_unp.called)
+        mock_get.assert_called_once_with(task, 'fake_vif_id')
         mock_clear.assert_called_once_with(self.port)
 
     @mock.patch.object(common_utils, 'warn_about_deprecated_extra_vif_port_id',
