@@ -684,7 +684,7 @@ class TestObject(_LocalTest, _TestObject):
 # version bump. It is an MD5 hash of the object fields and remotable methods.
 # The fingerprint values should only be changed if there is a version bump.
 expected_object_fingerprints = {
-    'Node': '1.22-f2c453dd0b42aec8d4833a69a9ac79f3',
+    'Node': '1.23-6bebf8dbcd2ce15407c946bd091f80b4',
     'MyObj': '1.5-9459d30d6954bffc7a9afd347a807ca6',
     'Chassis': '1.3-d656e039fd8ae9f34efc232ab3980905',
     'Port': '1.7-898a47921f4a1f53fcdddd4eeb179e0b',
@@ -692,21 +692,21 @@ expected_object_fingerprints = {
     'Conductor': '1.2-5091f249719d4a465062a1b3dc7f860d',
     'EventType': '1.1-aa2ba1afd38553e3880c267404e8d370',
     'NotificationPublisher': '1.0-51a09397d6c0687771fb5be9a999605d',
-    'NodePayload': '1.4-1ab0efe090ee3b2bd48d280a5acec1d4',
+    'NodePayload': '1.6-33b165d11b05f8e1a91da5b1bd344dda',
     'NodeSetPowerStateNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetPowerStatePayload': '1.4-9ab5116c2c59dfb9e56dcc7ef660a431',
+    'NodeSetPowerStatePayload': '1.6-b32af2903cba9a61be83f5b119188e4d',
     'NodeCorrectedPowerStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCorrectedPowerStatePayload': '1.4-d624154b8fb4e6cdda1554a8bf3f4cf5',
+    'NodeCorrectedPowerStatePayload': '1.6-e44d9d913afba61281190082605dd1fb',
     'NodeSetProvisionStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetProvisionStatePayload': '1.4-605c17b779aeedf0925937b2785d9f43',
+    'NodeSetProvisionStatePayload': '1.6-5e608497664b122ae53240f35072d731',
     'VolumeConnector': '1.0-3e0252c0ab6e6b9d158d09238a577d97',
     'VolumeTarget': '1.0-0b10d663d8dae675900b2c7548f76f5e',
     'ChassisCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'ChassisCRUDPayload': '1.0-dce63895d8186279a7dd577cffccb202',
     'NodeCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCRUDPayload': '1.2-b7a265a5e2fe47adada3c7c20c68e465',
+    'NodeCRUDPayload': '1.4-e55b77489d3ff71d561d0ae03668e298',
     'PortCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'PortCRUDPayload': '1.2-233d259df442eb15cc584fae1fe81504',
     'NodeMaintenanceNotification': '1.0-59acc533c11d306f149846f922739c15',
@@ -717,6 +717,8 @@ expected_object_fingerprints = {
     'VolumeConnectorCRUDPayload': '1.0-5e8dbb41e05b6149d8f7bfd4daff9339',
     'VolumeTargetCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'VolumeTargetCRUDPayload': '1.0-30dcc4735512c104a3a36a2ae1e2aeb2',
+    'Trait': '1.0-3f26cb70c8a10a3807d64c219453e347',
+    'TraitList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
 }
 
 
